import { 
  select, 
  csv, 
  scaleLinear,
  scaleTime,
  max,
  axisLeft,
  axisBottom,
  extent,
  format,
  timeParse,
  timeFormat,
  line
} from 'd3';

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const parseDate = timeParse("%d/%m/%Y");
const formatDate = timeFormat("%b %d");

const render = data => {
  const xValue = d => d.Date;
  const yValue = d => d.Depth;
  const circleRadius = 10;
  const margin = { top: 20, right:40, bottom:180, left: 150 };
  const title = 'GSI Borehole: Water levels by week';
  const xAxisLabel = 'Date';
  const yAxisLabel = 'Water level (mbgl)';
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

 const div = d3.select("body").append("div")	
    .attr("class", "tooltip")				
    .style("opacity", 0);
  
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight])
    .nice();
  
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(20);
  
  const xAxisTickFormat = data =>
    formatDate(data);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(20)
    .tickFormat(xAxisTickFormat);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.right})`);
  
  const yAxisG = g.append('g').call(yAxis);
  
  yAxisG.selectAll('.domain').remove();
  
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform',`translate(0,${innerHeight})`)
    .selectAll("text")
      .style("text-anchor", "end")
        .attr("dx", "-.8em")
        .attr("dy", ".15em")
        .attr("transform", "rotate(-65)");
  
  xAxisG.select('.domain').remove();
  
  g.append('text')
    .attr('class', 'title')
    .attr('y', -3)
    .attr('fill', 'black')
    .text(title);

  g.append('text')
    .attr('class','axis-label')
    .attr('y', innerHeight + 120)
    .attr('x', innerWidth /2 - 50)
    .attr('fill', 'black')
    .text(xAxisLabel);
  
  yAxisG.append('text')
    .attr('class','axis-label')
    .attr('y', -80)
    .attr('x', -innerHeight / 2)
    .attr('transform',`rotate(-90)`)
    .attr('fill', 'black')
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);
  
 const lineGenerator = line()
   .x(d => xScale(xValue(d)))
   .y(d => yScale(yValue(d)));
  
  
 g.append('path')
    .attr('class', 'line-path')
    .attr('d',lineGenerator(data));
  
  g.selectAll('circle')
    .data(data)
  	  .enter()
        .append('circle')
  		.attr('cy', d => yScale(yValue(d)))
        .attr('cx', d => xScale(xValue(d)))
        .attr('r', circleRadius)
    .on("mouseover", d => {	
            div.transition()		
                .duration(200)		
                .style("opacity", .9);		
            div	.html(`Date: ${formatDate(d.Date)} 
					   Depth: ${d.Depth}`)	
                .style("left", (d3.event.pageX) + "px")		
                .style("top", (d3.event.pageY - 28) + "px");	
  })					
    .on("mouseout", d => {		
        div.transition()		
            .duration(500)		
            .style("opacity", 0);	
    });
  
};

csv('https://vizhub.com/careyshan/datasets/borehole_dummydata1.csv')
  .then(data => {
    data.forEach(d => {
      console.log(data);
      d.Depth = +d.Depth;
      d.Date = parseDate(d.Date);
  })
  render(data);
});
